#!/usr/bin/env python
# ODE solver using Euler forward method
# (c) Peter Beerli
#
#
from pprint import *

class Vector(list):
    def __add__(self,other):
        v = [0]*len(self)
        for i in xrange(len(self)):
            v[i] = self[i] + other[i]
        return Vector(v)
    
    #def __rmul__(self,other):
    #    """ only works for two list arguments """
    #    return self*other

    def __mul__(self,other):
        """ only works for two list arguments"""
        v = [0] * len(self)
        for i in xrange(len(self)):
            v[i] = self[i] * other
        return Vector(v)

class ODE(object):
    def __init__(self, interval,initial0,maxiter,rhs_list,dt):
        self.y = [initial0]
        self.a,self.b = interval
        self.dt = dt
        self.t = 0
        self.maxiter = maxiter
        self.rhs = rhs_list
        self.dim = len(initial0)

        if(self.dim != len(self.rhs)):
            print "right hand side and initial values have inconsistent length",self.dim,len(self.rhs)
            raise Exception('Inconsistent Arguments')
        
    def solve(self,niter):
        """ advance solution n iterations"""
        iter = 0
        while self.t < self.b and iter < self.maxiter:
            rhs = Vector([self.rhs[i](self.y[-1],self.t) for i in xrange(self.dim)])
            ynew = self.y[-1] + rhs * self.dt
            self.y.append(ynew)
            iter += 1
            self.t += self.dt
        return self.y


if __name__ == "__main__":

    def f(y,t): 
        return(-y[1] +  f.a)

    def g(y,t): 
        return(y[0])

    interval = [0.,100.]
    initial0 = Vector([0.,1.0])
    maxiter  = 10000
    rhs_list      = Vector([f,g])
    dt       = 0.001
    param = 0.01

    for f in rhs_list:
        f.a = param

    import matplotlib.pyplot as plt

    fig1 = plt.subplot(211)

    mycolors = ['red', 'blue', 'green']
    c = 0
    
    ode = ODE(interval,initial0,maxiter,rhs_list,dt)
    solution = ode.solve(10000)
    ti = [dt*i for i in xrange(0,len(solution))]
    sol2 = zip(*solution)
    fig1.plot(ti,sol2[0],marker='', color=mycolors[0])
    fig1.plot(ti,sol2[1],marker='', color=mycolors[1])
    fig1.legend(['y1(t)','y2(t)'])
    fig1.set_ylabel('y1 and y2')
    fig1.set_xlabel('t')
    fig2 = plt.subplot(212)
    fig2.plot(sol2[0],sol2[1])
    fig2.axis('scaled')
    fig2.set_ylabel('y1')
    fig2.set_xlabel('y2')

    plt.show()
