#!/usr/bin/env python
# ODE solver using Euler forward method
# (c) Peter Beerli
#
#
from pprint import *

def ode(interval,initial0,maxiter,rhs,dt):
    """
    ode takes an interval an intial value, number of iterations, the right hand side, and a delta
    returns a list of y values
    """
    y = [initial0]
    a,b = interval
    t = 0
    iter = 0

    while t < b and iter < maxiter:
        ynew = y[-1] + dt * rhs(y[-1],t)
        y.append(ynew)
        iter += 1
        t += dt
    
    return y


if __name__ == "__main__":

    def rhs1(y,t): 
        return(-y)

    def rhs2(y,t): 
        return(-3.*y)

    def rhs3(y,t): 
        return(-15.*y)

    interval = [1,5]
    initial0 = 1.0
    maxiter  = 1000
    rhs      = [rhs1,rhs2,rhs1]
    dt       = 0.2

    import matplotlib.pyplot as plt

    fig = plt.figure()
    mycolors = ['red', 'blue', 'green']
    c = 0
    for rhsi in rhs:
        solution = ode(interval,initial0,maxiter,rhsi,dt)
        ti = [dt*i for i in xrange(0,len(solution))]
        sol = [[t,s] for t,s in zip(ti,solution)]
        plt.plot(ti,solution,marker='+', color=mycolors[c])
        c += 1
    plt.show()
